﻿package {
	// SodaReader.as - Actionscript 3 Version - Jim Bumgardner 2008

	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.ui.Keyboard;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;

	public class SodaReader
	{
		var parMC;
		var models = ["daintywalker.xml", "puppetpete.xml", "lamp.xml", "jello.xml","discodan.xml"];
		var curModel = 0;
		var myLoader:URLLoader;
		var but1, but2, s1, s2, s3;
		var title_txt:TextField;
		var s1c, s2c, s3c, b1c, b2c;
		
		public function SodaReader(parMC, title_txt, but1, but2, s1, s2, s3)
		{
			ss = new SpringSim(parMC, 3, 0.25, 0.01);
			ss.simFloor = 300;
			ss.setRenderFlags(SpringSim.RENDER_SPRINGS | SpringSim.RENDER_DOTS);
			title_txt.autoSize = TextFieldAutoSize.LEFT;
			
			myLoader = new URLLoader();
			myLoader.addEventListener(Event.COMPLETE, xmlLoadHandler);
			myLoader.load(new URLRequest(models[curModel]));
			this.title_txt = title_txt;
			this.but1 = but1;
			this.but2 = but2;
			this.s1 = s1;
			this.s2 = s2;
			this.s3 = s3;
			b1c = new ToggleButton(but1, 'show polys', false, togglePolys);
			b2c = new PlainButton(but2, 'next model', loadNextModel);
			s1c = new SliderController(s1, 'gravity',-2,2, ss.gravK,function(t) { ss.setGravity(t); } );
			s2c = new SliderController(s2, 'ground friction',0,1, 1-ss.floorDamping,function(t) { ss.setFloorRoughness(t); } );
			s3c = new SliderController(s3, 'wave speed',-.03,.03, ss.waveSpeed,function(t) { ss.waveSpeed = t; } );
		}

		function setupUI()
		{
			b1c.setState(false);
			ss.setRenderFlags(SpringSim.RENDER_SPRINGS | SpringSim.RENDER_DOTS); 
			s1c.setVal(ss.gravK);
			s2c.setVal(1-ss.floorDamping);
			s3c.setVal(ss.waveSpeed);
		}
		
		function togglePolys(t)
		{
			ss.setRenderFlags(t? (SpringSim.RENDER_DOTS | SpringSim.RENDER_POLYS) : (SpringSim.RENDER_SPRINGS | SpringSim.RENDER_DOTS)); 
		}
		
		function xmlLoadHandler(evt)
		{
			layerNbr = 100;
			ox = 0;
			oy = 200;
			tw = 1;
			th = -1;
			ss.reset();
			loadModel();
			setupUI();
		}
		
		function loadModel()
		{
			trace('loading model');
		    var myXML = XML(myLoader.data);
			var spritesByName = {};
			if (myXML.hasOwnProperty("settings")) {
				if (myXML.settings.@gravitydirection == 'up') {
					ss.setGravity(-ss.gravK);
				}
				if (myXML.settings.@wavedirection == 'forward') {
				}
				if (myXML.settings.@autoreverse == 'on')
					ss.setAutoReverse(true);
				else
					ss.setAutoReverse(false);
			}
			if (myXML.hasOwnProperty("name"))
			{
				title_txt.text = myXML.name;
			}
			if (myXML.hasOwnProperty("gridscale")) 
			{
				tw = Number(myXML.gridscale.@cellwidth);
				th = Number(myXML.gridscale.@cellheight);
				ox = Number(myXML.gridscale.@originx);
				oy = Number(myXML.gridscale.@originy);
			}
			if (myXML.hasOwnProperty("environment")) {
				if (myXML.environment.@gravity)
					ss.setGravity(Number(myXML.environment.@gravity));
				if (myXML.environment.@friction)
					ss.setFriction(Number(myXML.environment.@friction));
				if (myXML.environment.@springyness)
					ss.setStiffness(Number(myXML.environment.@springyness));
			}
			if (myXML.hasOwnProperty("collisions")) {
				if (myXML.collisions.@surface_friction)
					ss.setFloorRoughness(Number(myXML.collisions.@surface_friction));
			}
			if (myXML.hasOwnProperty("wave")) {
				var amp = Number(myXML.wave.@amplitude);
				var phase = Number(myXML.wave.@phase)*Math.PI*2;
				var speed = Number(myXML.wave.@speed);
				ss.setWave(amp,phase,speed);
			}
			for each (var mass:XML in myXML..mass)
			{
				var mc = ss.addDotMC(ox+tw*Number(mass.@x), oy+th*Number(mass.@y), (Number(mass.@fixed) == 1) );
				spritesByName[mass.@id] = mc;
			}
			for each (var poly:XML in myXML..poly)
			{
				var clr = poly.@color;
				if (clr.charAt(0) == '#')
					clr = Number('0x' + clr.substr(1));
				var ptNames = poly.@points.split(',');
				var curvy = Number(poly.@curvy);
				var pts = [];
				for (var i = 0; i < ptNames.length; ++i) 
				{
					pts.push(spritesByName[ptNames[i]]);
				}
				ss.addPolygon(pts,clr,curvy);
			}
			var a,b,spr;
			for each (var spring:XML in myXML..spring) // also 'muscle'
			{
				a = spring.@a;
				b = spring.@b;
		
				spr = ss.addSpring(spritesByName[a], spritesByName[b]);
				
				if (spring.@restlength > 0)
				{
					spr.restLen = spring.@restlength;
				}
				if (spring.@amplitude > 0) {
					amp = Number(spring.@amplitude);
					phase = Number(spring.@phase)*Math.PI*2;
					spr.setMuscle(amp,phase);
				}
			}
			for each (spring in myXML..muscle) // also 'muscle'
			{
				a = spring.@a;
				b = spring.@b;
		
				spr = ss.addSpring(spritesByName[a], spritesByName[b]);
				
				if (spring.@restlength > 0)
				{
					spr.restLen = spring.@restlength;
				}
				if (spring.@amplitude > 0) {
					amp = Number(spring.@amplitude);
					phase = Number(spring.@phase)*Math.PI*2;
					spr.setMuscle(amp,phase);
				}
			}
		}


		function loadNextModel()
		{
			trace(">> loadnextmodel");
			curModel = (curModel+1) % models.length;
			trace("loading " + models[curModel]);
			myLoader.load(new URLRequest(models[curModel]));
		}


	} // end class
} // end package